# -*- coding: utf-8 -*-
# kd_verbinder.py
"""entkoppelt vom grafischen Aspekt die Eigenschaften und Methoden
fuer Verbinder von Klassen in Klassendiagrammen"""


import platform
versionsNummer=platform.python_version()[0]
from verbinder import Verbinder
from verbinder1 import Verbinder1
from verbinder2 import Verbinder2

### ---------------------------------------------------
class KD_Verbinder(object):
    """Linie oder Pfeil fuer UML-Diagramme
    Verbinder wird wie eine abstrakte Klasse behandelt."""

    def __init__(self,
                 startKarte,
                 startKante,
                 zielKarte,
                 zielKante,
                 name=''):
        self.__name = name
        self.__startKarte=startKarte
        self.__startKante=startKante
        self.__startTeiler=50 # Prozent der Kante
        self.__zielKarte=zielKarte
        self.__zielKante=zielKante
        self.__zielTeiler=50 # Prozent der Kante
        self.__verbinder=None # der grafische Aspekt
        self.__verbindertyp=0
        self.__richtung=0 # fuer Pfeil zum Ziel
        self.__typ=0 #

    ## Get-Methoden
    def GibDaten(self): return self.GibStartWerte()+self.GibZielWerte()
    def GibNamen(self): return self.__name
    def GibStartKarte(self): return self.__startKarte
    def GibStartKante(self): return self.__startKante
    def GibStartTeiler(self): return self.__startTeiler
    def GibZielKarte(self): return self.__zielKarte
    def GibZielKante(self): return self.__zielKante
    def GibZielTeiler(self): return self.__zielTeiler
    def GibVerbinder(self): return self.__verbinder
    def GibDatenTextlich(self):
        '''gibt die Daten als Strings zurueck'''
        dat=self.GibStartKarte().GibNamen(),self.__startKante,str(self.__startTeiler),\
            self.GibZielKarte().GibNamen(),self.GibZielKante(),str(self.__zielTeiler),\
            self.GibNamen(),str(self.__typ),str(self.__verbindertyp),str(self.__richtung)
        return dat
    def GibTypNamen(self):
        if self.__typ==0: return 'Assoziation'
        if self.__typ==1: return 'Aggregation'
        if self.__typ==2: return 'Komposition'
        if self.__typ==3: return 'Vererbung'
        return None
   
    def SetzeVerbinderTyp(self, verbinderTyp=0):
        '''setzt den Typ des grafischen Aspekts
        0: Linie,  1: einfacher Winkel,  2: Doppelwinkel'''
        if self.__verbinder!=None:
            self.__verbinder.Verberge()
        self.__verbindertyp=verbinderTyp
        if self.__startKante==None: return
        if self.__zielKante==None: return
        x0,y0,x1,y1 = self.GibDaten()
        von_nach=self.__startKante+self.__zielKante
        if verbinderTyp==0:
            self.__verbinder=Verbinder(x0,y0,x1,y1,self.__name,
                                       self.__richtung,self.__typ)
        elif verbinderTyp==1:
            self.__verbinder=Verbinder1(x0,y0,x1,y1,self.__name,von_nach,
                                        self.__richtung,self.__typ)
        else:
            self.__verbinder=Verbinder2(x0,y0,x1,y1,self.__name,von_nach,
                                        self.__richtung,self.__typ)
        self.__verbinder.Zeige()

    def SetzeRichtung(self, richtung):
        '''ermoeglicht das Setzen einer Richtung, also einer Pfeilspitze
        Vererbung hat keine zusätzliche Richtung'''
        if self.__verbinder==None: return
        if self.__typ==3: return
        self.__richtung = richtung
        self.GibVerbinder().SetzeRichtung(richtung)
    
    def SetzeTyp(self, typ):
        '''ermoeglicht das Setzen von
        0: einfache Assoziation,
        1: Aggregation,
        2: Komposition oder
        3: Vererbung'''
        if self.__verbinder==None: return
        self.__verbinder.Verberge()
        self.__typ=typ
        self.__verbinder.SetzeTyp(typ)
        self.__verbinder.Zeige()

    def SetzeStartKarte(self, klassenkarte, kante=None):
        self.__startKarte=klassenkarte
        if kante!=None: self.__startKante=kante
        
    def SetzeStartKante(self, kante='N'):
        '''eine andere Kante verwenden'''
        if self.__startKarte==None: return
        self.__startKante=kante
        self.SetzeVerbinderTyp(self.__verbindertyp)
        self.UpdateVerbinder()
        
    def GibStartWerte(self):
        if self.__startKarte==None: return None
        if self.__startKante==None: return None
        x,y,b,h=self.__startKarte.GibAbmessungen()
        ##print('Start',x,y,b,h)
        if self.__startKante=='N':
            return x+b*self.__startTeiler/100, y
        elif self.__startKante=='S':
            return x+b*self.__startTeiler/100, y+h
        elif self.__startKante=='W':
            return x, y+h*self.__startTeiler/100
        else:                # 'O'
            return x+b, y+h*self.__startTeiler/100

    def SetzeZielKarte(self, klassenkarte, kante=None):
        self.__zielKarte=klassenkarte
        if kante!=None: self.__zielKante=kante
        
    def GibZielWerte(self):
        if self.__zielKarte==None: return None
        if self.__zielKante==None: return None
        x,y,b,h=self.__zielKarte.GibAbmessungen()
        ##print('Ziel',x,y,b,h)
        if self.__zielKante=='N':
            return x+b*self.__zielTeiler/100, y
        elif self.__zielKante=='S':
            return x+b*self.__zielTeiler/100, y+h
        elif self.__zielKante=='W':
            return x, y+h*self.__zielTeiler/100
        else:                # 'O'
            return x+b, y+h*self.__zielTeiler/100

    def SetzeZielKante(self, kante='N'):
        '''eine andere Kante verwenden'''
        if self.__zielKarte==None: return
        self.__zielKante=kante
        self.SetzeVerbinderTyp(self.__verbindertyp)
        self.UpdateVerbinder()
        
    def NeuerTeilerStartKante(self, teiler=50):
        '''setzt den prozentual Anteil auf der Kante neu'''
        self.__startTeiler=teiler
        self.UpdateVerbinder()

    def NeuerTeilerZielKante(self, teiler=50):
        '''setzt den prozentual Anteil auf der Kante neu'''
        self.__zielTeiler=teiler
        self.UpdateVerbinder()
        
    def NeuerName(self, name):
        '''fuegt dem Pfeil einen Namen hinzu'''
        pass ## T O D O

    def UpdateVerbinder(self):
        self.__verbinder.Verberge()
        self.__verbinder.SetzeStart(self.GibStartWerte())
        self.__verbinder.SetzeZiel(self.GibZielWerte())
        self.__verbinder.Zeige()

    def Verberge(self):
        self.__verbinder.Verberge()

    def Zeige(self):
        self.__verbinder.Zeige()
