# -*- coding: utf-8 -*-
# verbinder2.py

## rechtwinklige Verbindung mit zwei Winkeln

import platform
versionsNummer=platform.python_version()[0]
import math

from figuren import *
from verbinder import Verbinder

### ---------------------------------------------------
class Verbinder2(Verbinder):
    """Linie oder Pfeil fuer UML-Diagramme
    In dieser Variante werden drei zu einander senkrechte Linien
    verwendet
    Voraussetzung ist O->W, N->S oder umgekehrt"""

    def __init__(self,
                 x0, y0, x1, y1,
                 name='',
                 anschluss='OW',  # alternativ 'WO', 'NS' oder 'SN'
                 richtung=0, # Pfeil?
                 typ=0
                 ):
        self.__anschluss=anschluss
        ## die anderen Informationen ergeben sich aus den Koordinaten
        self.__teiler=0.5 # standardmaessig die Haelfte
        Verbinder.__init__(self,x0,y0,x1,y1,name,richtung,typ)
        # Figur als FigurenGruppe initialisieren

    def AendereTeiler(self,teiler):
        self.Verberge()
        self.__teiler=teiler
        self.Zeige()

    def GibFigur(self):
        '''gibt die aktuelle Figur zurueck'''
        x0,y0,x1,y1 = self.GibDaten()
        b,h=x1-x0,y1-y0
        figur=FigurenGruppe(x0,y0)
        if self.__anschluss in ('OW','WO'):
            xz_relativ=int((x1-x0)*self.__teiler) 
            figur.FuegeHinzu(Linie(0,0,xz_relativ,0))
            figur.FuegeHinzu(Linie(xz_relativ,0,0,h))
            figur.FuegeHinzu(Linie(xz_relativ,h,b-xz_relativ,0))
        else:   # 'NS' oder 'SN'
            yz_relativ=int((y1-y0)*self.__teiler)
            figur.FuegeHinzu(Linie(0,0,0,yz_relativ))
            figur.FuegeHinzu(Linie(0,yz_relativ,b,0))
            figur.FuegeHinzu(Linie(b,yz_relativ,0,h-yz_relativ))
        if self.__anschluss[1]=='N':
            if self.GibRichtung()==1: # Pfeil zeichnen
                figur.FuegeHinzu(
                    N_Eck(b,h,((-7,-15),(0,0),(7,-15)),
                          geschlossen=False))
            elif self.GibTyp()==3: # Vererbungspfeil
                figur.FuegeHinzu(
                    N_Eck(b,h,((-8,-20),(0,0),(8,-20)),
                          fuellfarbe='white'))
        elif self.__anschluss[1]=='S':
            if self.GibRichtung()==1: # Pfeil zeichnen
                figur.FuegeHinzu(
                    N_Eck(b,h,((-7,15),(0,0),(7,15)),
                          geschlossen=False))
            elif self.GibTyp()==3: # Vererbungspfeil
                figur.FuegeHinzu(
                    N_Eck(b,h,((-8,20),(0,0),(8,20)),
                          fuellfarbe='white'))
        elif self.__anschluss[1]=='W':
            if self.GibRichtung()==1: # Pfeil zeichnen
                figur.FuegeHinzu(
                    N_Eck(b,h,((-15,-7),(0,0),(-15,7)),
                          geschlossen=False))
            elif self.GibTyp()==3: # Vererbungspfeil
                figur.FuegeHinzu(
                    N_Eck(b,h,((-20,-8),(0,0),(-20,8)),
                          fuellfarbe='white'))
        else:                   # 'O'
            if self.GibRichtung()==1: # Pfeil zeichnen
                figur.FuegeHinzu(
                    N_Eck(b,h,((15,-7),(0,0),(15,7)),
                          geschlossen=False))
            elif self.GibTyp()==3: # Vererbungspfeil
                figur.FuegeHinzu(
                    N_Eck(b,h,((20,-8),(0,0),(20,8)),
                          fuellfarbe='white'))
        # Raute ?
        if not(self.GibTyp() in (1,2)): return figur
        if self.GibTyp()==1: ff='white'
        if self.GibTyp()==2: ff='black'
        if self.__anschluss[0]=='O':
            figur.FuegeHinzu(
                N_Eck(0,0,((0,0),(15,-5),(30,0),(15,5)),fuellfarbe=ff))
        elif self.__anschluss[0]=='W':
            figur.FuegeHinzu(
                N_Eck(0,0,((0,0),(-15,-5),(-30,0),(-15,5)),fuellfarbe=ff))
        elif self.__anschluss[0]=='S':
            figur.FuegeHinzu(
                N_Eck(0,0,((0,0),(-5,15),(0,30),(5,15)),fuellfarbe=ff))
        else:                   # 'N'
            figur.FuegeHinzu(
                N_Eck(0,0,((0,0),(-5,-15),(0,-30),(5,-15)),fuellfarbe=ff))
        return figur
            
### ---------------------------------------------------

class TestApp(wx.App):
    """Test-Anwendung"""
    def OnInit(self):
        self.__fenster = GrafikFenster(None, "Klassendiagramm")
        self.SetTopWindow(self.__fenster)
        self.__fenster.Show(True)
        self.__fenster.panel.Refresh()
        self.__fenster.ZeigeShellFrame()
        self.TestEinfach()
        #self.__fenster.ZeigeFillingFrame()
        return True
        
    def TestEinfach(self):
        '''isoliert die vier Faelle'''
        global v,v1,v2,v3,v4
        v=Verbinder2(120, 200, 180, 50, anschluss='OW')
        v2=Verbinder2(320, 200, 200, 50, anschluss='WO')
        v3=Verbinder2(310, 300, 380, 150,anschluss='NS')
        v3=Verbinder2(10, 100, 80, 350,anschluss='SN')

### ---------------------------------------------------
if __name__ == '__main__':
    app = TestApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()

